package de.lmu.ifi.mfchords.views;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;



import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PointF;
import android.graphics.Paint.Style;
import android.graphics.drawable.Drawable;
import android.media.MediaScannerConnection;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.support.v4.view.MotionEventCompat;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.LinearLayout.LayoutParams;
import de.lmu.ifi.mfchords.R;
import de.lmu.ifi.mfchords.utils.Constants;
import de.lmu.ifi.mfchords.utils.Finger;
import de.lmu.ifi.mfchords.utils.MFCListener;
import de.lmu.ifi.mfchords.utils.MFCRecognizer;

/**
 * This View takes touch events and classifies them. Do a calibration first to create the classifier. 
 * 
 */
public class DemoView extends LinearLayout implements MFCListener{

	private ArrayList<PointF> points = new ArrayList<PointF>();
	
	private Drawable nf_index;
	private Drawable nf_middle;
	private Drawable nf_ring;
	private Drawable tp_index;
	private Drawable tp_middle;
	private Drawable tp_ring;
	private Drawable ti_middle;
	private Drawable ti_ring;
	private Drawable ti_pinky;
	
	
	private Drawable prediction;
	LinkedHashMap<String,Drawable> predictions = new LinkedHashMap< String,Drawable>();
	
	private ImageView predictionImage;
	private TextView fingerTxt;
	
	//private String text = "Calibration has finished! Continue on to the Application";
	private RelativeLayout rl;
	
	//ArrayList<String> data = new ArrayList<String>();
	
	private Paint touchFeedbackPaint = new Paint();
	private Paint pTouchFeedbackPaint = new Paint();
	
	
	@SuppressLint("NewApi")
	public DemoView(Context context) {
		super(context);
		
		this.setWillNotDraw(false);
		
		
		MFCRecognizer recognizer = MFCRecognizer.getRecognizer(); 
		
		recognizer.addMFCRecognizer(this);
		recognizer.readInCalibration();
		recognizer.wantFeedforward(true);
		
		
		
		touchFeedbackPaint.setColor(Color.BLUE);
		touchFeedbackPaint.setStyle(Style.FILL);
		touchFeedbackPaint.setStrokeWidth(15);
		
		pTouchFeedbackPaint.setColor(Color.CYAN);
		pTouchFeedbackPaint.setStyle(Style.FILL);
		pTouchFeedbackPaint.setStrokeWidth(15);
		
		nf_index = getContext().getResources().getDrawable(R.drawable.nf_index);
		predictions.put(Constants.NF_INDEX,nf_index);
		
		nf_middle = getContext().getResources().getDrawable(R.drawable.nf_middle);
		predictions.put(Constants.NF_MIDDLE,nf_middle);
		
		nf_ring = getContext().getResources().getDrawable(R.drawable.nf_ring);
		predictions.put(Constants.NF_RING,nf_ring);
		
		ti_middle = getContext().getResources().getDrawable(R.drawable.ti_middle);
		predictions.put(Constants.TI_MIDDLE,ti_middle);
		
		ti_ring = getContext().getResources().getDrawable(R.drawable.ti_ring);
		predictions.put(Constants.TI_RING,ti_ring);
		
		ti_pinky = getContext().getResources().getDrawable(R.drawable.ti_pinky);
		predictions.put(Constants.TI_PINKY,ti_pinky);
		
		tp_index = getContext().getResources().getDrawable(R.drawable.tp_index);
		predictions.put(Constants.TP_INDEX,tp_index);
		
		tp_middle = getContext().getResources().getDrawable(R.drawable.tp_middle);
		predictions.put(Constants.TP_MIDDLE,tp_middle);
		
		tp_ring = getContext().getResources().getDrawable(R.drawable.tp_ring);
		predictions.put(Constants.TP_RING,tp_ring);
		
		LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		rl  = (RelativeLayout)inflater.inflate(R.layout.demo_layout, null);
		
		predictionImage = (ImageView)rl.findViewById(R.id.instruction);
		fingerTxt = (TextView)rl.findViewById(R.id.finger);
		fingerTxt.setText("");
		
		this.addView(rl);
		
		
	    invalidate();
		
	}
	
	@Override
	protected void dispatchDraw(Canvas canvas) {
		//canvas.drawColor(Color.YELLOW);
		super.dispatchDraw(canvas);
		
	}
	
	
	
	
	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		
		
		canvas.drawColor(Color.WHITE);
		
		
		/*for(PointF p: points) {
			canvas.drawCircle(p.x, p.y - 50 , 50, touchFeedbackPaint);
			
		}
		points.clear();*/
		
		
		for(Finger f : MFCRecognizer.getRecognizer().getHandModel().getAllFingers()){
			if(f!=null && f.isPositionSet()){
				canvas.drawCircle(f.position.x, f.position.y, 50, touchFeedbackPaint);
				
			}
		}
		
		
		for(Finger f : MFCRecognizer.getRecognizer().getHandModel().getAllFingers()){
			if(f!=null && f.isPredictedPositionSet()){
				canvas.drawCircle(f.getPredictedPosition().x, f.getPredictedPosition().y, 50, pTouchFeedbackPaint);
				
			}
		}
		
		
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		
		MFCRecognizer.getRecognizer().onTouchEvent(event);
		invalidate();
		return true;
	}
	
	@Override
	public void MFCRecognized(String mfc) {
		// TODO Auto-generated method stub
		
		predictionImage.setImageDrawable(predictions.get(mfc));
		predictionImage.setVisibility(View.VISIBLE);
		
	}
	
	@Override
	public void MFCReleased() {
		// TODO Auto-generated method stub
		fingerTxt.setVisibility(View.INVISIBLE);
		predictionImage.setVisibility(View.INVISIBLE);
		
	}
	
	@Override
	public void MFCFingerReleased(String f) {
		// TODO Auto-generated method stub
		
		if(f.equals(this.fingerTxt.getText())){
			fingerTxt.setVisibility(View.INVISIBLE);
		}
		
		
	}
	
	
	@Override
	public void MFCTapRecognized(String mfc, String tap) {
		// TODO Auto-generated method stub
		
		
		predictionImage.setImageDrawable(predictions.get(mfc));
		predictionImage.setVisibility(View.VISIBLE);
		
		this.fingerTxt.setText(tap);
		this.fingerTxt.setVisibility(View.VISIBLE);
		
	}
		
	
	
}
